import { _decorator, assetManager, Camera, Component, error, ImageAsset, instantiate, Label, math, native, Node, Prefab, RenderTexture, Size, Sprite, SpriteFrame, sys, Texture2D, UITransform, v3, Vec3, view } from 'cc';
import { GraphicsQRCode } from './GraphicsQRCode';
import { Canvas2Image } from './Canvas2Image';
const { ccclass, property } = _decorator;

@ccclass('TwoInviteQRCodePrefab')
export class TwoInviteQRCodePrefab extends Component {

    @property(Node)
    root: Node = null!;
    @property(Node)
    title: Node = null!;
    @property(Node)
    tip: Node = null!;
    @property(Node)
    copy: Node = null!;
    @property(Camera)
    rootCamera: Camera = null!;

    @property(Prefab)
    qrcodePrefab: Prefab = null!;

    rt: RenderTexture = null;
    _buffer: ArrayBufferView = null!;
    _canvas: HTMLCanvasElement = null!;
    canvas2image: Canvas2Image = null!;

    start() {

        this.initView();
        this.initClampTexture();
    }

    initView () {
        this.setTipString("分享二维码，扫一扫加入对战");

        const titleTransform = this.title.getComponent(UITransform);
        const tipTransform = this.tip.getComponent(UITransform);
        const titleQRCodeGap: number = 50, tipQRCodeGap: number = 20;
        const qrcodeTopPosY: number = this.title.position.y - titleTransform.height*0.5 - titleQRCodeGap;
        const qrcodeMaxHeight: number = qrcodeTopPosY - (this.tip.position.y + tipTransform.height*0.5 + tipQRCodeGap);
        
        const rootTransform = this.root.getComponent(UITransform);
        const widthGap: number = 50;
        const qrcodeMaxWidth: number = rootTransform.width - widthGap*2;

        const qrcodeWH: number = qrcodeMaxHeight > qrcodeMaxWidth?qrcodeMaxWidth:qrcodeMaxHeight;

        const qrcode = instantiate(this.qrcodePrefab);
        this.root.addChild(qrcode);
        const qrcodeTS = qrcode.getComponent(GraphicsQRCode);
        qrcodeTS.createQRCode(new Size(qrcodeWH, qrcodeWH));

        qrcode.setPosition(v3(0, qrcodeTopPosY - qrcodeMaxHeight*0.5, 0));
    }

    setTipString (text: string) {
        const tipLabel = this.tip.getComponent(Label);
        tipLabel.string = text;
    }

    initClampTexture () {
        this.canvas2image = Canvas2Image.getInstance();
        this.rt = new RenderTexture();
        let rtSize = new math.Size;
        rtSize.width = Math.round(view.getVisibleSize().width);
        rtSize.height = Math.round(view.getVisibleSize().height);
        this.rt.reset(rtSize);
        this.rootCamera.targetTexture = this.rt;

        this.scheduleOnce(() => {
            this.beginClampTexture();
        }, 0.1);
    }

    beginClampTexture () {
        const rootTransform = this.root.getComponent(UITransform);
        var worldPos = this.root.getWorldPosition();
        this._buffer = this.rt.readPixels(Math.round(worldPos.x-rootTransform.width*0.5), Math.round(worldPos.y-rootTransform.height*0.5), Math.round(rootTransform.width), Math.round(rootTransform.height));
        // this.showImage(rootTransform.width, rootTransform.height);
    }

    // showImage(width: number, height: number) {
    //     let img = new ImageAsset();
    //     img.reset({
    //         _data: this._buffer,
    //         width: width,
    //         height: height,
    //         format: Texture2D.PixelFormat.RGBA8888,
    //         _compressed: false
    //     });
    //     let texture = new Texture2D();
    //     texture.image = img;
    //     let sf = new SpriteFrame();
    //     sf.texture = texture;
    //     sf.packable = false;
    //     this.copy!.getComponent(Sprite).spriteFrame = sf;
    //     this.copy!.getComponent(Sprite).spriteFrame.flipUVY = true;
    //     if (sys.isNative && (sys.os === sys.OS.IOS || sys.os === sys.OS.OSX)) {
    //         this.copy!.getComponent(Sprite).spriteFrame.flipUVY = false;
    //     }
    //     const copyTransform = this.copy?.getComponent(UITransform);
    //     copyTransform?.setContentSize(new Size(width, height));
    // }

    onSaveQRCodeClicked () {
        this.setTipString("等待好友加入对战");
        
        const rootTransform = this.root.getComponent(UITransform);
        this.savaAsImage(rootTransform.width, rootTransform.height, this._buffer);
        
    }

    savaAsImage(width: number, height: number, arrayBuffer: any) {
        let w = Math.floor(width);
        let h = Math.floor(height)
        if (sys.isBrowser) {
            if (!this._canvas) {
                this._canvas = document.createElement('canvas');
                this._canvas.width = w;
                this._canvas.height = h;
            } else {
                this.clearCanvas();
            }
            let ctx = this._canvas.getContext('2d')!;
            let rowBytes = w * 4;
            for (let row = 0; row < h; row++) {
                let sRow = h - 1 - row;
                let imageData = ctx.createImageData(w, 1);
                let start = sRow * w * 4;
                for (let i = 0; i < rowBytes; i++) {
                    imageData.data[i] = arrayBuffer[start + i];
                }
                ctx.putImageData(imageData, 0, row);
            }
            
            //@ts-ignore
            this.canvas2image.saveAsPNG(this._canvas, w, h);
            // this.tips.string = `保存图片成功`;
            this.showTipPopup("保存图片成功");

        } else if (sys.isNative) {
            // console.log("原生平台暂不支持图片下载");
            // return;
            let filePath = native.fileUtils.getWritablePath() + 'render_to_sprite_image.png';

            // 目前 3.0.0 ~ 3.4.0 版本还不支持 jsb.saveImageData ,引擎计划在 3.5.0 支持, 要保存 imageData 为本地 png 文件需要参考下方的 pr 定制引擎
            // https://gitee.com/zzf2019/engine-native/commit/1ddb6ec9627a8320cd3545d353d8861da33282a8

            //@ts-ignore
            if (native.saveImageData) {
                //@ts-ignore
                let success = native.saveImageData(this._buffer, width, height, filePath);
                if (success) {
                    // 用于测试图片是否正确保存到本地设备路径下
                    assetManager.loadRemote<ImageAsset>(filePath, (err, imageAsset)=> {
                        if (err) {
                            console.log("show image error")
                        } else {
                            var newNode = instantiate(this.root);
                            newNode.setPosition(new Vec3(-newNode.position.x, newNode.position.y, newNode.position.z));
                            this.root.parent.addChild(newNode);
                            
                            const spriteFrame = new SpriteFrame();
                            const texture = new Texture2D();
                            texture.image = imageAsset;
                            spriteFrame.texture = texture;
                            newNode.getComponent(Sprite).spriteFrame = spriteFrame;
                            spriteFrame.packable = false;
                            spriteFrame.flipUVY = true;
                            if (sys.isNative && (sys.os === sys.OS.IOS || sys.os === sys.OS.OSX)) {
                                spriteFrame.flipUVY = false;
                            }
    
                            // this.tips.string = `成功保存在设备目录并加载成功: ${filePath}`;
                            this.showTipPopup("成功保存在设备目录并加载成功: " + `${filePath}`);
                        }
                    });
                    console.log("save image data success, file: " + filePath);
                    // this.tips.string = `成功保存在设备目录: ${filePath}`;
                    this.showTipPopup("成功保存在设备目录: " + `${filePath}`);
                }
                else {
                    error("save image data failed!");
                    // this.tips.string = `保存图片失败`;
                    this.showTipPopup("保存图片失败");
                }
            }
        } else if (sys.platform === sys.Platform.WECHAT_GAME) {
            if (!this._canvas) {
                //@ts-ignore
                this._canvas = wx.createCanvas();
                this._canvas.width = width;
                this._canvas.height = height;
            } else {
                this.clearCanvas();
            }
            var ctx = this._canvas.getContext('2d');

            var rowBytes = width * 4;

            for (var row = 0; row < height; row++) {
                var sRow = height - 1 - row;
                var imageData = ctx.createImageData(width, 1);
                var start = sRow * width * 4;

                for (var i = 0; i < rowBytes; i++) {
                    imageData.data[i] = arrayBuffer[start + i];
                }

                ctx.putImageData(imageData, 0, row);
            }
            //@ts-ignore
            this._canvas.toTempFilePath({
                x: 0,
                y: 0,
                width: this._canvas.width,
                height: this._canvas.height,
                destWidth: this._canvas.width,
                destHeight: this._canvas.height,
                fileType: "png",
                success: (res) =>{
                    //@ts-ignore
                    wx.showToast({
                        title: "截图成功"
                    });
                    // this.tips.string = `截图成功`;
                    this.showTipPopup("截图成功");
                    //@ts-ignore
                    wx.saveImageToPhotosAlbum({
                        filePath: res.tempFilePath,
                        success: (res)=> {
                            //@ts-ignore              
                            wx.showToast({
                                title: "成功保存到设备相册",
                            });
                            // this.tips.string = `成功保存在设备目录: ${res.tempFilePath}`;
                            this.showTipPopup("成功保存在设备目录: " + `${res.tempFilePath}`);
                        },
                        fail: ()=> {
                            // this.tips.string = `保存图片失败`;
                            this.showTipPopup("保存图片失败");
                        }
                    })
                },
                fail: ()=> {
                    //@ts-ignore
                    wx.showToast({
                        title: "截图失败"
                    });
                    // this.tips.string = `截图失败`;
                    this.showTipPopup("截图失败");
                }
            })
        }
    }

    clearCanvas() {
        let ctx = this._canvas.getContext('2d');
        ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
    }
    
    showTipPopup (contentText: string) {
        alert(contentText);
    }
}


